/*****************************************************************************
 *  main.c:  main() PFC - PFC Sistemas Empotrados
 *
 *   Copyright(C) 2014, Jesús Nicolás
 *   All rights reserved.
 *
 *
******************************************************************************/

// Includes
#include "FreeRTOS.h"
#include "task.h"

#include <stdio.h>
#include "queue.h"
#include "semphr.h"

#include "LPC17xx.h"
#include "uart.h"
#include "printf.h"
#include "acelerometro.h"
#include "wifly.h"
#include "config.h"
#include <led.h>

// Longitud de cola
#define mainQUEUE_LENGTH				( 1 )

// Tareas del sistema Si3
void vGestionLeds( void *pvParameters );
void vGestionWifly( void *pvParameters );
void vGestionAcelerometro( void *pvParameters );

typedef struct {
	int origen;
	int led;
} Led_Reg;

int estado_led = ESTADO_INICIAL;

//  Gestion de tareas y colas

// Handle
xTaskHandle xHandle_Leds;
xTaskHandle xHandle_Acce;
xTaskHandle xHandle_Wifly;

// Cola de cambio de leds
xQueueHandle  xCambioLedQueue;

// Semáforo de control
xSemaphoreHandle xControlIOSemaphore;




/*****************************************************************************
** Function name:		IniciaRecursos
**
** Descriptions:		Crea el semaforo y la cola para gestionar el sistema
** 						y coordinar las tareas productoras y consumidoras
** 						e inicia los leds y acelerómetro
**
** parameters:
** Returned value:		TRUE or FALSE success
**
*****************************************************************************/
uint32_t IniciaRecursos(void){

//  Crea la cola de tareas y semaforos para la gestion de los procesos
	Print_T_CRLF(UART_CON," > Iniciando Tareas y Semaforos");
	xControlIOSemaphore = xSemaphoreCreateMutex();
	xCambioLedQueue = xQueueCreate(mainQUEUE_LENGTH, sizeof( Led_Reg ) );

	uint32_t Led = LED_1_MASK | LED_2_MASK | LED_3_MASK | LED_4_MASK;

	if(( xCambioLedQueue == NULL ) || ( xControlIOSemaphore==NULL ) ) //JS
		return(FALSE);
	else {
		// Inicialización de LEDS
		    	Print_T_CRLF(UART_CON," > Iniciando LEDS");
		    	LED_Initialize(Led);
		    	//Inicialización del acelerometro con resolución baja (0)
			   	if (Inicia_Acelerometro(0) == TRUE) {
		    		Print_T_CRLF(UART_CON," > Acelerometro MMA7361 Iniciado correctamente");
			     	return (TRUE);
			   	}
			    else {
		   	    	Print_T_CRLF(UART_CON,"Error al inicializar el acelerometro MMA7361");
		   	    	return(FALSE);
			    }
	}

}


int main( void )
{
	{
	    UART_Init(UART_CON, BR_1152);
	    UART_Flush (UART_CON);
	    UART_Init(UART_WIFLY, BR_1152);
	    Delay_Ms(Brute, 2000);

	    Print_T_CRLF(UART_CON,"INICIANDO SISTEMA");

		if (SysTick_Config(SystemCoreClock / 1000)) {
			  Print_T_CRLF(UART_CON,"Error al inicializar SysTick Time");
			  while (1);
		}

		if (!IniciaRecursos()) Print_T_CRLF(UART_CON,"Error al inicializar medicion acelerometro Sistema");
		else {

			if(( xCambioLedQueue != NULL )&& (xControlIOSemaphore !=NULL))		{
				// Tareas:  Puntero a la funcion, nombre de la tarea, Tamaño pila, paso de parametros , prioridad,  handle de la tarea
				xTaskCreate( vGestionWifly,(signed char *) "Tarea gestion Wifly ", 512, NULL, 3, &xHandle_Wifly);
				xTaskCreate( vGestionAcelerometro,(signed char *)"Tarea gestion Acelerometro",512, NULL, 3, &xHandle_Acce);
				xTaskCreate( vGestionLeds,(signed char *)"Tarea gestion leds",512, NULL, 2, &xHandle_Leds);
				//Inicia el gestor de tareas
		        Print_T_CRLF(UART_CON, (char *)" --------------    PLANIFICADOR LANZADO   --------------");
		    	Print_T_CRLF(UART_CON, (char *)" ");
				vTaskStartScheduler();
			}
			else {
				if (xControlIOSemaphore==NULL) Print_T_CRLF(UART_CON,"El semaforo de control  no se ha podido crear");
				else Print_T_CRLF (UART_CON,"La cola xCambioLedQueue para envio de datos no se ha podido crear");
		}
	}

}
	
	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
	return 0;
}

/*-----------------------------------------------------------*/

/*****************************************************************************
** Task name:			vGestionWifly
**
** Descriptions:		La función de esta tarea es la de realizar las lecturas
** 						de los datos que se reciben por el WiFly, escribiendo en
** 						la cola xCambioLedQueue un registro con el nuevo estado
**
*****************************************************************************/

void vGestionWifly ( void *pvParameters ){



    char *key = CLAVEB;
    int ledRemoto = 0;
    Led_Reg rec_cambio;
    Print_T_CRLF(UART_CON, (char *)" +++++++++++++++ T A R E A vGestionWifly +++++++++++++++");

    portTickType xTicksToWait = 100 / portTICK_RATE_MS; //n

#ifdef AUTOWIFION
    // La autoconfiguración de la LAN solo se ejecuta con AUTOWIFION activado en config.h
    // Busca el SSID en todos los canales WIFI y si no lo encuentra se autoconfigura como APNODE
    // Si lo encuentra se confiugura como CLIENTE
    // Si no está activado cada equipo arranca según tenga configurado su WiFly (Más rápido)
    Wifly_Auto_Rol(UART_WIFLY, SSID);
#endif
    Delay_Ms(Task, 1000);

	for( ;; ) {
		Print_T_CRLF(UART_CON, (char *)"\r\n (W) *** BUCLE TAREA G. WIFLY ***");
		if ((xSemaphoreTake (xControlIOSemaphore, (portTickType) portMAX_DELAY ) ==pdTRUE )) {
			ledRemoto = Wifly_Udp_Read(UART_WIFLY, key);
			if (ledRemoto != 0) {
				rec_cambio.origen = O_REMOTO;
				rec_cambio.led = ledRemoto;
				xQueueSendToBack( xCambioLedQueue, &rec_cambio, xTicksToWait );
				estado_led = ledRemoto;
			}
#ifdef MODE_DEBUG
			Print_TN_CRLF(UART_CON, (char *)"\r\n (W) ----------- Wifly led -->", ledRemoto);
#endif
			xSemaphoreGive(xControlIOSemaphore );//Libera el semaforo
			Delay_Ms(Task, 10);
		} else
		    	Print_T_CRLF(UART_CON, " (W) Imposible obtener el semaforo de xControlIOSemaphore" );
		taskYIELD();

	}
}


/*****************************************************************************
** Task name:			vGestionAcelerometro
**
** Descriptions:		Esta tarea se encarga de tomar lecturas del acelerómetro
** 						(la media de 100 capturas) de las coordenadas X,Y,Z cada
** 						cierto tiempo y de escribir un registro en la cola cambios
**                      de leds
**
*****************************************************************************/
void vGestionAcelerometro( void *pvParameters){

    A_Reg rec_Posicion;
    Led_Reg rec_cambio;
	float rep_X, rep_Y,rep_Z;
    int reposo_prev = true;
    int cont = 0;

	Print_T_CRLF(UART_CON, (char *)" +++++++++++++++ T A R E A vGestionAcelerometro ++++++++");
    portTickType xTicksToWait = 100 / portTICK_RATE_MS;



  // Medidas en  estado de reposo para calibración
    Read_Acelerometro(500,&rec_Posicion);
    rep_X =  rec_Posicion.Lectura_X;
    rep_Y =  rec_Posicion.Lectura_Y;
    rep_Z =  rec_Posicion.Lectura_Z;

#ifdef AUTOWIFION
    Delay_Ms(Task, 12000); // En espera de la autoconfiguración de LAN
#endif

    Delay_Ms(Task, 1000);
 	LED_Switch(LED_1_MASK, true);
 	LED_Switch(LED_2_MASK, true);
 	LED_Switch(LED_3_MASK, true);
 	LED_Switch(LED_4_MASK, true);
 	Delay_Ms(Task, 1000);
 	LED_Switch(LED_1_MASK, false);
 	LED_Switch(LED_2_MASK, false);
 	LED_Switch(LED_3_MASK, false);
 	LED_Switch(LED_4_MASK, false);
    // Dispositivo Listo

	for( ;; ) {
		Print_T_CRLF(UART_CON, (char *)"\r\n (A) *** BUCLE TAREA G. ACELEROMETRO ***");
#ifdef MODE_DEBUG
		Print_TN_CRLF (UART_CON," (A) ESTADO ACTUAL: ",estado_led );
#endif
		Read_Acelerometro(100,&rec_Posicion);

		cont++;
		if ((rec_Posicion.Lectura_X > rep_X + Ace_offset) ||
			(rec_Posicion.Lectura_X < rep_X - Ace_offset) ||
			(rec_Posicion.Lectura_Y > rep_Y + Ace_offset) ||
			(rec_Posicion.Lectura_Y < rep_Y - Ace_offset) ||
			(rec_Posicion.Lectura_Z > rep_Z + Ace_offset) ||
			(rec_Posicion.Lectura_Z < rep_Z - Ace_offset))
		{
#ifdef MODE_DEBUG
			Print_TN_CRLF (UART_CON,"\r\n (A) >>LECTURA DE LA COLA ACELEROMETRO EN SW-------------X------------>",rec_Posicion.Lectura_X );
			Print_TN_CRLF (UART_CON," (A) >>LECTURA DE LA COLA ACELEROMETRO EN SW-------------Y------------>",rec_Posicion.Lectura_Y );
			Print_TN_CRLF (UART_CON," (A) >>LECTURA DE LA COLA ACELEROMETRO EN SW-------------Z------------>",rec_Posicion.Lectura_Z );
#endif
			if  (reposo_prev == true) {
				reposo_prev = false;
		// Manejo de estados de los leds
				switch ( estado_led )
				{
				  case ESTADO_1:
					  estado_led=ESTADO_2;
				  break;
				  case ESTADO_2:
					  estado_led=ESTADO_3;
				  break;
				  case ESTADO_3:
					  estado_led=ESTADO_4;
				  break;
				  case ESTADO_4:
					  estado_led = ESTADO_5;
				  break;
				  case ESTADO_5:
					  estado_led = ESTADO_1;
				  break;
				  default:
				  break;
				}
			rec_cambio.origen = O_LOCAL;
			rec_cambio.led = estado_led;
#ifdef MODE_DEBUG
			Print_TN_CRLF (UART_CON,"\r\n (A) >>LECTURA DE LA COLA ACELEROMETRO----------------X------------------------->",rec_Posicion.Lectura_X );
			Print_TN_CRLF (UART_CON," (A) >>LECTURA DE LA COLA ACELEROMETRO----------------Y------------------------->",rec_Posicion.Lectura_Y );
			Print_TN_CRLF (UART_CON," (A) >>LECTURA DE LA COLA ACELEROMETRO----------------Z------------------------->",rec_Posicion.Lectura_Z );
			Print_TN_CRLF (UART_CON," (A) >>ITERACION LECTURA ACELEROMETRO------------------------------------->",cont );
			Print_TN_CRLF(UART_CON, (char *)"\r\n (A) --- TAMAÑO COLA EN ACELERO ANTES --->", uxQueueMessagesWaiting(xCambioLedQueue));
#endif
			xQueueSendToBack( xCambioLedQueue, &rec_cambio, xTicksToWait );
			}
		}
		else {
#ifdef MODE_DEBUG
			Print_TN_CRLF (UART_CON,"\r\n(A) >>LECTURA DE LA COLA ACELEROMETRO EN REPOSO----------------X--------------->",rec_Posicion.Lectura_X );
			Print_TN_CRLF (UART_CON,"(A) >>LECTURA DE LA COLA ACELEROMETRO EN REPOSO----------------Y--------------->",rec_Posicion.Lectura_Y );
			Print_TN_CRLF (UART_CON,"(A) >>LECTURA DE LA COLA ACELEROMETRO EN REPOSO----------------Z--------------->",rec_Posicion.Lectura_Z );
#endif

			if (rec_Posicion.Lectura_Y < rep_Y + Ace_sen  &&
				rec_Posicion.Lectura_Y > rep_Y - Ace_sen) {
					reposo_prev = true;
			}
		}

		Delay_Ms(Task, 500); // Toma muestras cada 500 ms
		taskYIELD();
	}
}


/*-----------------------------------------------------------*/

/*****************************************************************************
** Task name:			vGestionLeds
**
** Descriptions:		La misión de esta tarea es la de esperar a recibir un
** 						registro de cambio de led en la cola xCambioLedQueue,
** 						y a continuación, cambiar los led al
** 						nuevo estado y enviar un registro con dicho cambio a los
** 						dispositivos remotos.
** 						Tambien procesa los cambios remotos escritos por la tarea
** 						vGestionWifly cambiando los leds de estado
**
*****************************************************************************/

void vGestionLeds ( void *pvParameters ){


	Led_Reg rec_cambio;
    int cont = 0;
    int ciclo = 0;
	int max_cic = MAX_CICLOS;
	char *key = CLAVEB;
	char *dispositivo = DISPOSITIVO;

    Print_T_CRLF(UART_CON, (char *)" +++++++++++++++ T A R E A vGestionLeds ++++++++++++++++");

#ifdef AUTOWIFION
    Delay_Ms(Task, 12000); // En espera de la autoconfiguración de LAN
#endif

    Delay_Ms(Task, 2000);

	for( ;; ) {
		if ((xSemaphoreTake (xControlIOSemaphore, (portTickType) portMAX_DELAY ) ==pdTRUE )) {
			Print_T_CRLF(UART_CON, (char *)"\r\n (L) *** BUCLE TAREA G. LEDS ***");
			if ( xQueueReceive( xCambioLedQueue, &rec_cambio, 0 ) ==pdTRUE )
			{
				cont++;
				switch ( rec_cambio.led )	{
					case 1:
						LED_Switch(LED_1_MASK, true);
						LED_Switch(LED_2_MASK, false);
						LED_Switch(LED_3_MASK, false);
						LED_Switch(LED_4_MASK, false);
						if (rec_cambio.origen == O_LOCAL) {
							Wifly_Udp_Send(UART_WIFLY, key, dispositivo,ESTADO_1);
						}
						ciclo = 0;
					break;
					case 2:
						LED_Switch(LED_1_MASK, false);
						LED_Switch(LED_2_MASK, true);
						LED_Switch(LED_3_MASK, false);
						LED_Switch(LED_4_MASK, false);
						if (rec_cambio.origen == O_LOCAL) {
							Wifly_Udp_Send(UART_WIFLY, key, dispositivo,ESTADO_2);
						}
						ciclo = 0;
					break;
					case 3:
						LED_Switch(LED_1_MASK, false);
						LED_Switch(LED_2_MASK, false);
						LED_Switch(LED_3_MASK, true);
						LED_Switch(LED_4_MASK, false);
						if (rec_cambio.origen == O_LOCAL) {
							Wifly_Udp_Send(UART_WIFLY, key, dispositivo,ESTADO_3);
						}
						ciclo = 0;
					break;
					case 4:
						LED_Switch(LED_1_MASK, false);
						LED_Switch(LED_2_MASK, false);
						LED_Switch(LED_3_MASK, false);
						LED_Switch(LED_4_MASK, true);
						if (rec_cambio.origen == O_LOCAL) {
							Wifly_Udp_Send(UART_WIFLY, key, dispositivo,ESTADO_4);
						}
						ciclo = 0;
					break;
					case 5:
						LED_Switch(LED_1_MASK, false);
						LED_Switch(LED_2_MASK, false);
						LED_Switch(LED_3_MASK, false);
						LED_Switch(LED_4_MASK, false);
						if (rec_cambio.origen == O_LOCAL) {
							Wifly_Udp_Send(UART_WIFLY, key, dispositivo,ESTADO_5);
						}
						ciclo = 0;
						break;
						default:
						break;
					}
			}
			else {
#ifdef MODE_DEBUG
			Print_T_CRLF(UART_CON," (L) Timeout de espera cola Leds");
#endif
			}

			ciclo++;

			if (ciclo == max_cic) {
				Wifly_Udp_Send(UART_WIFLY, key, dispositivo,0);
				Wifly_SW_Broadcast(UART_WIFLY, MAX_INTER);
				ciclo = 0;
			}
			xSemaphoreGive(xControlIOSemaphore );//Libera el semaforo
			} else
			    	Print_T_CRLF(UART_CON, " (L) Imposible obtener el semaforo de xControlIOSemaphore" );
		}
}

/*-----------------------------------------------------------*/

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	Print_T_CRLF(UART_CON, (char *)"Error de Malloc Failed");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	Print_T_CRLF(UART_CON, (char *)"Error de Stack Overflow");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}




