/*****************************************************************************
 *   adc.h:  Header file for NXP LPC17xx Family Microprocessors
 *
 *   Copyright(C) 2009, NXP Semiconductor
 *   All rights reserved.
 *   Modify by: Jesus Nicolas Jaro - UOC (2014)
 *
 *   History
 *   2009.05.25  ver 1.00    Preliminary version, first Release
 *
******************************************************************************/

#ifndef __ADC_H 
#define __ADC_H

/* Para activar DMA mode, BURST mode and ADC_INTERRUPT flag deben estar a 1 */
/* Si In BURST mode, ADC_INTERRUPT need to be set. */
#define ADC_INTERRUPT_FLAG	0	/* 1 is interrupt driven, 0 is polling */
#define BURST_MODE			0   /* Burst mode works in interrupt driven mode only. */

#define ADC_OFFSET          0x10
#define ADC_INDEX           4

#define ADC_DONE            0x80000000
#define ADC_OVERRUN         0x40000000
#define ADC_ADINT           0x00010000

#define ADC_NUM				8
//Clock a 1Mhz
#define ADC_CLK				1000000


//Estructura para lectura tres canales ADC
typedef struct {
	int channel0;
	int channel1;
	int channel2;
	int channel3;
	int channel4;
	int channel5;
	int channel6;
	int channel7;
} ADC_reg_channels;


// Función que inicializa ADC
extern uint32_t ADCInit( uint32_t ADC_Clk );

// Función que lee el canal indicado
extern uint32_t ADCRead( uint8_t channelNum );

extern void ADCBurstRead( void );

// Función que que lee los 3 canales (a0-->x, a1-->y, a2 -->z) mediante una estructura
void ADCReadChannels(int N_Canales, ADC_reg_channels *Datos );

#endif /* end __ADC_H */

/*****************************************************************************
**                            End Of File
******************************************************************************/
