/**
 * @file led.c
 * @brief Implementación del driver LED.
 */
#include <FreeRTOS.h>
#include <LPC17xx.h>

#include <led.h>
#include <config.h>
#include "printf.h"

/**
 * Inicializa los leds para mostrar la información de depuración
 * \param pinMask Los leds que se encenderán.
 */
void LED_Initialize(uint16_t pinMask)
{
	uint8_t i;

	for(i = 0; i < 16; i++)
	{
		if(pinMask & 1 << i)
		{
	//		Print_TN_CRLF (UART_CON,"\r\nINICIALIZO LED -------->",i );
			LPC_PINCON->PINSEL0	&= (~(3 << i * 2));
			LPC_GPIO0->FIODIR |= (1 << i);
		}
	}
}

/**
 * Enciende o apaga los leds seleccionados.
 * \param pinMask La máscara con los leds que serán encendidos.
 * \param value Indica si el led estará encendido o apagado.
 */
void LED_Switch(uint16_t pinMask, bool value)
{
	uint8_t i;

	for(i = 0; i < 16; i++)
	{
		if(pinMask & 1 << i)
		{
			//LOG("Switching %s led %d", value ? "on" : "off", i);

			if(value)
				LPC_GPIO0->FIOSET |= (1 << i);
			else
				LPC_GPIO0->FIOCLR |= (1 << i);
		}
	}
}
