/**
 \brief LPC17XX-specific definition of the "timer" bsp module.

 TIMER 0 and 1 are used by BSP timers
 TIMER 2 is virtualized to offer 100 timers.
 TIMER 3 is radiotimer
 \author Xavi Vilajosana <xvilajosana@eecs.berkeley.edu>, February 2012.
 */

#include "stdint.h"
#include "timer.h"
#include "LPC17xx.h"


//=========================== defines =========================================

extern void TIMER2_IRQHandler(void);

//=========================== variables =======================================

//callback that will be called when timer 2 expires.. this is called from the ISR function of timer2
low_timer_hook timer_compare_isr_hook_2;

//=========================== prototypes ======================================

//=========================== public ==========================================

void timer_set_isr_compare_hook(uint8_t timer_num, low_timer_hook cbt) {
	switch (timer_num) {
	case TIMER_NUM2:
		timer_compare_isr_hook_2 = cbt;
		break;
	default:
		while (1)
			; //this should never happen
		break;
	}
	return;
}

/**
 \brief Initialize a timer.
 */
void timer_init(uint8_t num_timer) {

	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;

	//power Timer/Counter 2 module [bit 22]
	LPC_SC ->PCONP |= (1 << 22);

	/*TODO ADD YOUR CODE HERE
	 1-configure peripheral clock - see PCLKSEL1 register (Table 41)
	 2-reset timer counter
	 3-configure preescaler
	 4-configure timer so it resets on match
	 */

	//enable timer2 interrupt in the NVIC
	NVIC_EnableIRQ(TIMER2_IRQn);
}

/**
 \brief Starts a timer.

 \pre Call timer_init before calling this function.
 */
void timer_enable(uint8_t num_timer) {
	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;
	//TODO
}

uint32_t timer_get_current_value(uint8_t num_timer) {
	uint32_t current = 0;
	//only timer 2
	if (num_timer != TIMER_NUM2)
		while (1)
			; //failure.. should neve happen
	//TODO
	return current;
}

/**
 \brief Set the compare value in the timer.

 Every timer has 3 compare registers. so choose one.
 */
void timer_set_compare(uint8_t num_timer, uint8_t compareReg,
		uint32_t delayInTicks) {

	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;
	//TODO

}

void timer_reset_compare(uint8_t num_timer, uint8_t compareReg) {
	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;
	//TODO

}
/**
 \brief Reset a timer.
 */
void timer_reset(uint8_t num_timer) {
	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;
	//TODO
}

/**
 \brief Disable a timer.
 */
void timer_disable(uint8_t num_timer) {
	//only timer 2
	if (num_timer != TIMER_NUM2)
		return;
	//TODO
}

/**
 \brief irq handler for timer 2.

 "implements" weak function in cr_startup_lpc17.c
 */
void TIMER2_IRQHandler(void) {

	if (LPC_TIM2 ->IR & (0x1 << 0)) {
		// clear interrupt flag
		LPC_TIM2 ->IR = 0x1 << 0;
		// call the callback
		timer_compare_isr_hook_2(TIMER_COMPARE_REG0);
	}
	if (LPC_TIM2 ->IR & (0x1 << 1)) {
		// clear interrupt flag
		LPC_TIM2 ->IR = 0x1 << 1;
		// call the callback
		timer_compare_isr_hook_2(TIMER_COMPARE_REG1);
	}
	if (LPC_TIM2 ->IR & (0x1 << 2)) {
		// clear interrupt flag
		LPC_TIM2 ->IR = 0x1 << 2;
		// call the callback
		timer_compare_isr_hook_2(TIMER_COMPARE_REG2);
	}
	return;
}
